package org.branulf.show_enchants.mixin;

import net.minecraft.client.font.TextRenderer;
import net.minecraft.client.gui.DrawContext;
import net.minecraft.client.util.math.MatrixStack;
import net.minecraft.component.DataComponentTypes;
import net.minecraft.component.type.ItemEnchantmentsComponent;
import net.minecraft.enchantment.Enchantment;
import net.minecraft.enchantment.Enchantments;
import net.minecraft.item.ItemStack;
import net.minecraft.item.Items;
import net.minecraft.registry.entry.RegistryEntry;
import net.minecraft.registry.tag.EnchantmentTags;
import net.minecraft.text.Text;
import net.minecraft.registry.RegistryKey;
import net.minecraft.util.Identifier;
import org.spongepowered.asm.mixin.Final;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

import java.util.List;
import java.util.Optional;
import java.util.regex.Pattern;
import com.google.common.collect.Lists;

@Mixin(DrawContext.class)
public abstract class DrawContextMixin {

    private static final Pattern COLOR_CODE_PATTERN = Pattern.compile("§[0-9A-FK-ORa-fk-or]");
    private static final Pattern INVISIBLE_CHARS = Pattern.compile("[\\p{Cf}\\p{Co}\\p{Cn}\\u2006]");

    @Shadow @Final private MatrixStack matrices;

    @Shadow
    public abstract int drawText(TextRenderer textRenderer, Text text, int x, int y, int color, boolean shadow);

    @Inject(
            method = "drawStackOverlay(Lnet/minecraft/client/font/TextRenderer;Lnet/minecraft/item/ItemStack;IILjava/lang/String;)V",
            at = @At("TAIL")
    )
    private void onRenderItemOverlay(TextRenderer renderer, ItemStack stack, int x, int y, String countLabel, CallbackInfo ci) {
        if (stack == null) return;

        // 处理附魔书
        if (stack.getItem() == Items.ENCHANTED_BOOK) {
            handleEnchantedBook(stack, renderer, x, y);
            return;
        }

        // 处理武器和工具
        handleWeaponsAndTools(stack, renderer, x, y);
    }

    private void handleEnchantedBook(ItemStack stack, TextRenderer renderer, int x, int y) {
        ItemEnchantmentsComponent enchants = stack.get(DataComponentTypes.STORED_ENCHANTMENTS);
        if (enchants == null || enchants.isEmpty()) return;

        List<RegistryEntry<Enchantment>> valid = Lists.newArrayList(enchants.getEnchantments());
        if (valid.isEmpty()) return;

        int index = (int) ((System.currentTimeMillis() / 1000) % valid.size());
        RegistryEntry<Enchantment> enchant = valid.get(index);

        String name = enchant.value().description().getString();
        String firstChar = getFirstVisibleChar(name);

        if (firstChar.isEmpty()) {
            firstChar = getFallbackChar(enchant);
        }

        if (!firstChar.isEmpty()) {
            matrices.push();
            matrices.translate(0, 0, 200);

            int color = enchant.isIn(EnchantmentTags.CURSE) ? 0xFFFF5555 :
                    enchant.isIn(EnchantmentTags.TREASURE) ? 0xFF55FFFF : 0xFFFFFFFF;

            this.drawText(renderer, Text.literal(firstChar), x + 2, y + 9, color, true);
            matrices.pop();
        }
    }

    private void handleWeaponsAndTools(ItemStack stack, TextRenderer renderer, int x, int y) {
        ItemEnchantmentsComponent enchants = stack.get(DataComponentTypes.ENCHANTMENTS);
        if (enchants == null || enchants.isEmpty()) return;

        List<RegistryEntry<Enchantment>> enchantments = Lists.newArrayList(enchants.getEnchantments());
        if (enchantments.isEmpty()) return;

        // 优先显示武器的主要附魔
        RegistryEntry<Enchantment> primaryEnchant = findPrimaryEnchantment(enchantments);
        if (primaryEnchant == null) return;

        String name = primaryEnchant.value().description().getString();
        String firstChar = getFirstVisibleChar(name);

        if (firstChar.isEmpty()) {
            firstChar = getFallbackChar(primaryEnchant);
        }

        if (!firstChar.isEmpty()) {
            matrices.push();
            matrices.translate(0, 0, 200);

            int color = primaryEnchant.isIn(EnchantmentTags.CURSE) ? 0xFFFF5555 :
                    primaryEnchant.isIn(EnchantmentTags.TREASURE) ? 0xFF55FFFF : 0xFFFF55FF;

            this.drawText(renderer, Text.literal(firstChar), x + 2, y + 9, color, true);
            matrices.pop();
        }
    }

    private RegistryEntry<Enchantment> findPrimaryEnchantment(List<RegistryEntry<Enchantment>> enchantments) {
        // 武器
        for (RegistryEntry<Enchantment> entry : enchantments) {
            Optional<RegistryKey<Enchantment>> key = entry.getKey();
            if (key.isPresent()) {
                Identifier id = key.get().getValue();
                if (id.equals(Enchantments.SHARPNESS.getValue()) ||
                        id.equals(Enchantments.SMITE.getValue()) ||
                        id.equals(Enchantments.BANE_OF_ARTHROPODS.getValue())) {
                    return entry;
                }
            }
        }

        // 工具
        for (RegistryEntry<Enchantment> entry : enchantments) {
            Optional<RegistryKey<Enchantment>> key = entry.getKey();
            if (key.isPresent()) {
                Identifier id = key.get().getValue();
                if (id.equals(Enchantments.SILK_TOUCH.getValue()) ||
                        id.equals(Enchantments.FORTUNE.getValue())) {
                    return entry;
                }
            }
        }

        // 杂七杂八
        for (RegistryEntry<Enchantment> entry : enchantments) {
            Optional<RegistryKey<Enchantment>> key = entry.getKey();
            if (key.isPresent()) {
                Identifier id = key.get().getValue();
                if (id.equals(Enchantments.RIPTIDE.getValue()) ||
                        id.equals(Enchantments.LOYALTY.getValue()) ||
                        id.equals(Enchantments.CHANNELING.getValue())) {
                    return entry;
                }
            }
        }

        return null;
    }


    private String getFirstVisibleChar(String str) {
        if (str == null || str.isEmpty()) {
            return "";
        }

        str = COLOR_CODE_PATTERN.matcher(str).replaceAll("");
        str = INVISIBLE_CHARS.matcher(str).replaceAll("").trim();

        if (str.isEmpty()) {
            return "";
        }

        int[] codePoints = str.codePoints().toArray();
        for (int codePoint : codePoints) {
            if (!Character.isISOControl(codePoint) &&
                    !Character.isWhitespace(codePoint) &&
                    !isPrivateUseCharacter(codePoint)) {
                return new String(Character.toChars(codePoint));
            }
        }
        return "";
    }

    private boolean isPrivateUseCharacter(int codePoint) {
        return (codePoint >= 0xE000 && codePoint <= 0xF8FF) ||
                (codePoint >= 0xF0000 && codePoint <= 0xFFFFD) ||
                (codePoint >= 0x100000 && codePoint <= 0x10FFFD);
    }

    private String getFallbackChar(RegistryEntry<Enchantment> enchant) {
        Optional<RegistryKey<Enchantment>> key = enchant.getKey();
        if (key.isPresent()) {
            String id = key.get().getValue().getPath();
            if (!id.isEmpty()) {
                return id.substring(0, 1).toUpperCase();
            }
        }
        return "?";
    }
}
